function updateupstreamvel(self, PrevOpRot, Mod)
    % UPDATEUPSTREAMVEL Update upstream velocities to account for previous rotor downwash.
    %   This function updates the axial and tangential upstream velocity of the current elements by
    %   taking into account the downwash of the previous rotor.
    %
    % Notes:
    %   At the moment, only the Simplified Multiple Stream Tube (SMST) model is implemented. It
    %   assumes:
    %     - no pre-contraction caused by the second rotor
    % -----
    %
    % Syntax:
    %   ElPerf.updateupstreamvel(PrevOpRot) updates upstream velocities in an ElemPerf object
    %   according to the performance of the previous Operating Rotor `PrevOpRot`.
    %
    % Inputs:
    %   prevOpRot : previous Operating Rotor object
    %
    % Outputs:
    %   Updated values for upstreamVelAx and upstreamVelTg for the current ElemPerf object
    %
    % See also: ElemPerf.
    %
    % <a href="https:/gitlab.uliege.be/rotare/documentation">Complete documentation (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Lyraie Rakotondratsimba
    % Co-Author/Maintainer: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/rotare/rotare
    % Docs: https://gitlab.uliege.be/rotare/documentation
    % Issues: https://gitlab.uliege.be/rotare/rotare/-/issues
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Map the velocities according to the appropriate mode
    switch Mod.coax.model
        case 'none'
            Mod.coax.contraType = 'none';
            prevWake = calccontraction(PrevOpRot, Mod.coax);
            mapvelocities(self, prevWake, length(self.Rot.Bl.y));
        case 'sst'
            sst(self, PrevOpRot, Mod.coax);
        case 'mst'
            mst(self, PrevOpRot, Mod.coax, false);
        case 'smst'
            mst(self, PrevOpRot, Mod.coax, true);
    end

end

% ==================================== Helper functions ============================================

function prevWake = calccontraction(PrevOpRot, ModCoax)
    % CALCCONTRACTION Calculates the wake characteristics of the first rotor

    % Velocity at the disk of previous rotor
    prevRot.velAx = PrevOpRot.ElPerf.upstreamVelAx + PrevOpRot.ElPerf.indVelAx;
    prevRot.velTg = PrevOpRot.ElPerf.indVelTg;
    prevRot.upstreamVel = PrevOpRot.ElPerf.upstreamVelAx;
    if strcmpi(ModCoax.model, 'sst')
        prevRot.velAx = mean(prevRot.velAx);
        prevRot.velTg = 0;
        prevRot.upstreamVel = mean(PrevOpRot.ElPerf.upstreamVelAx);
    end

    % Velocities in the wake of previous rotor after contraction
    switch ModCoax.contraType
        case 'none'
            prevWake.velAx = prevRot.velAx;
            prevWake.velTg = prevRot.velTg;
            prevWake.areaR = 1;
        case 'farfield'
            prevWake.velAx = 2 * prevRot.velAx - prevRot.upstreamVel;
            prevWake.velTg = 2 * prevRot.velTg; % FIXME?
            prevWake.areaR = prevRot.velAx ./ prevWake.velAx;
            % prevWake.VelTg = PrevOpRot.ElPerf.dQ ./ (PrevOpRot.Rot.Bl.y .* ...
            %                  PrevOpRot.ElPerf.massflow) .* (1 ./ areaRatio);
        case 'value'
            prevWake.areaR = Mod.coax.contraVal;
            prevWake.velAx = prevRot.velAx ./ prevWake.areaR;
            prevWake.velTg = prevRot.velTg ./ prevRot.velTg;
    end

    % This assumes annulus contraction, but not thickness contraction as well
    % It must be recalculated in general MST model
    prevWake.y = PrevOpRot.Rot.Bl.y .* sqrt(prevWake.areaR);
    prevWake.dy = PrevOpRot.Rot.Bl.dy .* sqrt(prevWake.areaR);
end

function sst(self, PrevOpRot, ModCoax)
    % SSTMODEL Implement the Simple Stream Tube model.
    %   Only consider the mean axial velocity and the entire streamtube contraction

    prevWake = calccontraction(PrevOpRot, ModCoax);
    prevWake.spinDir = self.Rot.Bl.spinDir * PrevOpRot.Rot.Bl.spinDir;

    % Resize properly and discard the swirl velocity
    prevWake.velAx = ones(size(prevWake.y)) * prevWake.velAx;
    prevWake.velTg = zeros(size(prevWake.y));

    % Map velocities
    vcRadIdx = calccradius(prevWake.y, self.Rot.Bl.y);
    mapvelocities(self, prevWake, vcRadIdx);

    % Check if massflows are conserved
    massflowcheck(self, PrevOpRot, prevWake, vcRadIdx, ModCoax);

end

function mst(self, PrevOpRot, ModCoax, simple)
    % MSTMODEL Implement the Multiple Stream Tube model.
    %   Consider individual annulus contraction, as well as the effect on the swirl velocity

    % Base calculation with only annulus contraction, no tube narrowing
    prevWake = calccontraction(PrevOpRot, ModCoax);
    prevWake.spinDir = self.Rot.Bl.spinDir * PrevOpRot.Rot.Bl.spinDir;

    if ~simple
        % Contract both annulus and tube thickness
        yPrevExt = zeros(size(PrevOpRot.Rot.Bl.y)); % External radius of previous tube
        yPrevExt(1) = PrevOpRot.Rot.Bl.y(1) - PrevOpRot.Rot.Bl.dy(1) / 2; % Hub

        yw = zeros(size(PrevOpRot.Rot.Bl.y)); % Annulus center in wake
        dyw = zeros(size(PrevOpRot.Rot.Bl.y)); % Annulus thickness in wake
        for i = 1:numel(PrevOpRot.Rot.Bl.y)
            if i > 1
                yPrevExt(i) = yw(i - 1) + dyw(i - 1) / 2;
            end

            dyw(i) = -yPrevExt(i) + sqrt(yPrevExt(i)^2 + ...
                                         2 * PrevOpRot.Rot.Bl.y(i) .* PrevOpRot.Rot.Bl.dy .* ...
                                         prevWake.areaR(i));

            yw(i) = yPrevExt(i) + dyw(i) / 2;
        end
        prevWake.y = yw;
        prevWake.dy = dyw;

    end

    % Map velocities
    vcRadIdx = calccradius(prevWake.y, self.Rot.Bl.y);
    mapvelocities(self, prevWake, vcRadIdx);

    % Check if massflows are conserved
    massflowcheck(self, PrevOpRot, prevWake, vcRadIdx, ModCoax);
end

function vcIdxOnLower = calccradius(vcY, y)
    % CALCVCRADIUS Calculate the radius of the vena contracta and its index on lower rotor

    vcIdxOnLower = find(y <= vcY(end), 1, 'last');
end

function mapvelocities(self, prevWake, vcRadIdx)
    % MAPVELOCITIES Map the velocities of the previous rotor wake onto the current rotor

    self.upstreamVelAx(1:vcRadIdx) = interp1(prevWake.y, prevWake.velAx, ...
                                             self.Rot.Bl.y(1:vcRadIdx), 'spline', 0);
    self.upstreamVelTg(1:vcRadIdx) = interp1(prevWake.y, prevWake.spinDir * prevWake.velTg, ...
                                             self.Rot.Bl.y(1:vcRadIdx), 'spline', 0);
end

function massflowcheck(self, PrevOpRot, prevWake, vcRadIdx, ModCoax)
    % MASSFLOWCHECK Verify if the massflow is conserved after inteprolation in the vena contracta
    %   The following three massflows should be roughly equal:
    %     1. Mass flow accros entire previous rotor disk
    %     2. Mass flow in wake after contraction of slipstream
    %     3. Mass flow on lower rotor elements inside vena contracta after interpolation of wake

    % 1. Entire previous rotor disk
    if strcmpi(ModCoax.model, 'sst')
        mf_prevRot = trapz(2 * pi * PrevOpRot.Rot.Bl.y .* PrevOpRot.Rot.Bl.dy .* ...
                           PrevOpRot.Op.Flow.rho .* ...
                           mean(PrevOpRot.ElPerf.upstreamVelAx + PrevOpRot.ElPerf.indVelAx));
    else
        mf_prevRot = PrevOpRot.totalmassflow;
    end

    % 2. Contracted wake
    mf_prevWake = trapz(2 * pi * prevWake.y .* prevWake.dy .* PrevOpRot.Op.Flow.rho .* ...
                        prevWake.velAx);
    % 3. Lower rotor elements inside vena contracta
    mf_curRot = trapz(2 * pi * self.Rot.Bl.y(1, 1:vcRadIdx) .* self.Rot.Bl.dy .* ...
                      PrevOpRot.Op.Flow.rho .* self.upstreamVelAx(1:vcRadIdx));

    % Error in mass flow conservation
    err_prevRot = (mf_prevRot - mf_prevWake) / mf_prevRot;  % prevRot disk/wake
    err_intrpCurr = (mf_prevWake - mf_curRot) / mf_prevWake; % prevRot wake/current vena contracta

    %     close all;
    %     figure
    %     hold on
    %     plot(PrevOpRot.Rot.Bl.y, PrevOpRot.ElPerf.upstreamVelAx + PrevOpRot.ElPerf.indVelAx)
    %     plot(prevWake.y, prevWake.velAx)
    %     plot(self.Rot.Bl.y, self.upstreamVelAx,'-o')
    %     legend('upper', 'vc','lower')
    %     pause

    % Throw error if mass flow not conserved
    assert(abs(err_prevRot) < 1e-12); % Should be 0 in theory
    assert(abs(err_intrpCurr) < 2e-2); % Should be small

end
