function [filtered, idx] = filter(self, alt, speed, rpm, coll)
    % FILTER Filter results based on operating point.
    %   This method isolate the OperRotor instances for each solver that corresponds to the
    %   operating point.
    %
    % -----
    %
    % Syntax:
    %   [filtered, idx] = Result.filter(alt, speed, rpm, coll) Return the Results of the four
    %   solvers for the operating point defined by `alt`, `rpm`, `coll`, `speed`.
    %
    % Inputs:
    %   alt     : Altitude, [m]
    %   rpm     : Rotational speed, [rpm]
    %   coll    : Collective pitch, [deg]
    %   speed   : Axial velocity, [m/s]
    %
    % Outputs:
    %   filtered : Result instance with only one instance for leishman, indfact, indevl and stahlhut
    %              properties.
    %   idx      : Index of the simulation corresponding to the operating point
    %
    % Example:
    %   ResRot.filter( 0, 0, 1250, 5)
    %
    % See also: rotare, Result, template.
    %
    % <a href="https://gitlab.uliege.be/rotare/documentation">Complete documentation (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: Needs adjustments for multiple rotors
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/rotare/rotare
    % Docs: https://gitlab.uliege.be/rotare/documentation
    % Issues: https://gitlab.uliege.be/rotare/rotare/-/issues
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    DEF.ALLOWED_SOLVERS = {'leishman', 'indfact', 'indvel', 'stahlhut'};

    % Input validation
    validateattributes(alt, {'numeric'}, {'scalar'});
    validateattributes(speed, {'numeric'}, {'scalar'});
    validateattributes(rpm, {'numeric'}, {'scalar'});
    validateattributes(coll, {'numeric'}, {'scalar'});

    function idx = getidx(type, val)
        idx = self.operPts.(type) == val | isnan(val);
    end

    iAlt = getidx('altitude', alt);
    iSpeed = getidx('speed', speed);
    iRpm = getidx('rpm', rpm);
    iColl = getidx('collective', coll);

    idx = iAlt & iColl & iRpm & iSpeed;

    % Filter for the solvers only (rest is useless or redundant here)
    for iSolv = 1:length(DEF.ALLOWED_SOLVERS)
        thissolv = DEF.ALLOWED_SOLVERS{iSolv};
        if ~isempty(self.(thissolv))
            filtered.(thissolv) = self.(thissolv)(idx, :);
        else
            filtered.(thissolv) = self.(thissolv);
        end
    end

end
