function save(self, SaveOpts)
    % SAVE Method to save the Result object to a MAT-file.
    %   This method saves the Result object in a MAT-file, following the user's the user's
    %   configured rules for the file naming and the saving directory.
    % -----
    %
    % Syntax:
    %   Result.save() Prompt the user for a file name and directory and then save it.
    %
    %   Result.save(SaveOpts) Save the result object and name it following the options passed in
    %   SaveOpts.
    %
    % Inputs:
    %   SaveOpts : (optional) Options for the naming of the saved file. Sturcture with fields
    %               - overwrite (bool) Overwrite previous result if filename is the same
    %               - dir       (char) Directory where the results are saved
    %               - filename  (char) Base file name of the saved result
    %
    % Outputs:
    %   MAT-file with the saved Result object.
    %
    % See also: rotare, Result, template.
    %
    % <a href="https://gitlab.uliege.be/rotare/documentation">Complete documentation (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/rotare/rotare
    % Docs: https://gitlab.uliege.be/rotare/documentation
    % Issues: https://gitlab.uliege.be/rotare/rotare/-/issues
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    DEF.OVERWRITE = false;
    DEF.FILENAME = 'RotareResult.mat';

    if nargin < 2
        % Prompt user for filename
        [filename, SaveOpts.dir] = uiputfile(DEF.FILENAME, '*.mat');
        filename = removeextension(filename); % Remove ext as it is added during save()
    else
        % Create directory for the results if it does not exist
        if ~isfolder(SaveOpts.dir)
            mkdir(SaveOpts.dir);
        end

        % Generate proper name based on user inputs
        filename = formatfilename(SaveOpts);
    end

    % Save to file
    tempFile.Results = self; % So it can be saved properly
    save([SaveOpts.dir, filename, '.mat'], '-struct', 'tempFile');

end

function filename = formatfilename(SaveOpts)
    % FORMATFILENAME Format the filename using user configuration preferences.

    TIME_FORMAT = 'YYYYmmddTHHMM';

    % Ensure user did not put an extension already
    filename = removeextension(SaveOpts.filename);

    % Prevent overwriting file
    if ~SaveOpts.overwrite
        % First preprend a timestamp at the front of the filename
        timestamp = datestr(datetime('now'), TIME_FORMAT);
        filename = [timestamp, '-', filename];

        % If not enough, add a _v1 at the end of the filename
        baseFilename = filename;
        i = 0;
        while isfile([SaveOpts.dir, filename, '.mat'])
            i = i + 1;
            filename = [baseFilename, '_v', num2str(i)];
        end
    end

end

function filename = removeextension(filename)
    % REMOVEEXTENSION Remove extension from a filename if there was one

    [path, file, ~] = fileparts(filename);
    filename = fullfile(path, file);

end
